@@ -24,35 +24,44 @@ class Gui:
     shoe_progress: tkinter.Label
     fix_mistakes: tkinter.IntVar
     slider: tkinter.Scale
+    insurance_chip: tkinter.Label
+    dealer_info: tkinter.Label
 
 
 class Game:
 
-    def __init__(self, player: Player, dealer: Dealer, gui: Gui, bet: int = 1):
+    def __init__(self, player: Player, dealer: Dealer, gui: Gui, args: any):
         self.player = player
         self.dealer = dealer
         self.gui = gui
-        self.bet = bet
+        self.args = args
+        self.bet = args.bet
         self.shoe = self.init_shoe()
         self.active_slot = None
-        self.initial_bet = bet
+        self.initial_bet = args.bet
 
     def deal(self):
         """Starts new round."""
         self.bet = self.gui.slider.get()
         self.gui.slider.configure(state=tkinter.DISABLED)
         self.hide_all_chips()
+        self.hide_insurance_chip()
         self.hide_fingers()
         self.clean_player_slots()
+        self.dealer_info()
         self.player.init_count()
         self.player.hands = []
         if self.shoe.n_cards < 52:
             self.shoe = Shoe(6)
         hand = self.player.start_new_hand(self.bet)
         self.dealer.init_hand()
+        if self.args.dealer_cards is not None:
+            self.shoe.arrange(self.args.dealer_cards)
         self.dealer.deal(self.shoe, self.gui.shoe_progress)
         self.dealer.deal(self.shoe, self.gui.shoe_progress)
         self.display_dealer_cards()
+        if self.args.cards is not None:
+            self.shoe.arrange(self.args.cards)
         hand.deal(self.shoe, self.gui.shoe_progress)
         hand.deal(self.shoe, self.gui.shoe_progress)
         self.show_buttons()
@@ -62,13 +71,19 @@ class Game:
         self.display_stack()
         self.enable_correct_buttons(hand)
         self.display_chip(hand, 0)
-        if hand.is_blackjack:
-            self.resolve_blackjack()
-            self.display_player_cards(hand, rotate_last=True)
+        self.display_player_cards(hand)
+        if self.dealer.cards[0].label != 'A':
+            self.hide_buttons(('insurance', 'even-money'))
+            if hand.is_blackjack:
+                self.resolve_blackjack()
         else:
-            self.display_player_cards(hand)
-        if self.dealer.cards[0].label == 'A':
-            self.hide_buttons(('surrender', ))
+            self.hide_buttons(('surrender',))
+            if hand.is_blackjack is True:
+                self.show_buttons(('even-money', ))
+                self.hide_buttons(('insurance', ))
+            else:
+                self.show_buttons(('insurance',))
+                self.hide_buttons(('even-money', ))
 
     def surrender(self):
         """Method for Surrender button."""
@@ -79,6 +94,13 @@ class Game:
         self.display_stack()
         self.deal()
 
+    def even_money(self):
+        """Method for Even Money button"""
+        self.dealer.even_money = True
+        hand = self.get_hand_in_active_slot()
+        self.hide(hand)
+        self.payout()
+
     def double(self):
         """Method for Double button."""
         hand = self.get_hand_in_active_slot()
@@ -122,14 +144,11 @@ class Game:
                 return
         self.hide_buttons(('surrender', 'double'))
         hand.deal(self.shoe, self.gui.shoe_progress)
-        if hand.sum == 21:
-            self.display_player_cards(hand, rotate_last=True)
-            hand.is_finished = True
-        else:
-            self.display_player_cards(hand)
+        self.display_player_cards(hand)
         if hand.is_over is True:
             self.hide(hand)
             self.hide_chips(hand)
+            self.display_info(hand, 'BUST')
         if hand.is_finished is False:
             self.enable_correct_buttons(hand)
         else:
@@ -144,6 +163,15 @@ class Game:
         hand.is_finished = True
         self.resolve_next_hand()
 
+    def insurance(self):
+        """Method for Insurance button."""
+        hand = self.get_hand_in_active_slot()
+        self.dealer.insurance_bet = (hand.bet / 2)
+        self.display_insurance_chip()
+        self.player.stack -= self.dealer.insurance_bet
+        self.display_stack()
+        self.hide_buttons(('insurance',))
+
     def split(self):
         """Method for Split button."""
         hand = self.get_hand_in_active_slot()
@@ -151,44 +179,26 @@ class Game:
             if self.check_play(hand, 'split') is False:
                 return
         self.hide_buttons(('surrender',))
-        n_hands = len(self.player.hands)
-        for ind in range(n_hands):
-            hand = self.player.hands[ind]
-            if hand.cards[0].value == hand.cards[1].value:
-                new_hand = self.player.start_new_hand(self.bet)
-                split_card = hand.cards.pop()
-                new_hand.deal(split_card, self.gui.shoe_progress)
-                hand.is_split_hand = True
-                new_hand.is_split_hand = True
-                self.display_chip(new_hand, 0)
-                self.display_stack()
-                for handy in (hand, new_hand):
-                    handy.deal(self.shoe, self.gui.shoe_progress)
-                    handy.is_split_hand = True
-                    if handy.sum == 21:
-                        handy.is_finished = True
-                    if handy.cards[0].label == 'A':
-                        # Split Aces receive only one card more
-                        handy.is_hittable = False
-                        handy.is_finished = True
-                self.player.hands[ind] = hand
-                break
+        new_hand = self.player.start_new_hand(self.bet)
+        split_card = hand.cards.pop()
+        new_hand.deal(split_card, self.gui.shoe_progress)
+        self.display_chip(new_hand, 0)
+        self.display_stack()
+        for handy in (hand, new_hand):
+            handy.is_split_hand = True
+            handy.deal(self.shoe, self.gui.shoe_progress)
+            if handy.cards[0].label == 'A':
+                # Split Aces receive only one card more
+                handy.is_hittable = False
+                handy.is_finished = True
 
-        self.player.hands.sort(key=lambda x: not x.cards[0].value == x.cards[1].value)
-        self.display_player_hands()
-        n_hands = len(self.player.hands)
-        for ind in range(n_hands):
-            hand = self.player.hands[ind]
-            if hand.cards[0].value == hand.cards[1].value and len(self.player.hands) < 4:
-                self.gui.menu['split']['state'] = tkinter.NORMAL
-                self.display_finger(hand)
-                self.enable_correct_buttons(hand)
-                break
-            else:
-                self.player.sort_hands()
-                self.display_player_hands()
-                self.resolve_next_hand()
-        self.display_player_hands()
+        self.player.sort_hands()
+        if len(self.player.hands) < 4:
+            self.player.hands.sort(key=lambda x: not x.cards[0].value == x.cards[1].value)
+        for hand in self.player.hands:
+            rotate = True if hand.cards[0].label == 'A' and hand.cards[1].label != 'A' else False
+            self.display_player_cards(hand, rotate_last=rotate)
+        self.resolve_next_hand()
 
     def resolve_next_hand(self):
         """Moves to next unfinished hand."""
@@ -199,8 +209,13 @@ class Game:
             self.display_finger(hand)
         else:
             self.clean_info()
-            if self.is_all_over() is False:
+            if self.is_all_over() is False or self.dealer.insurance_bet > 0:
                 self.display_dealer_cards(hide_second=False)
+                if self.dealer.is_blackjack and self.dealer.insurance_bet > 0:
+                    self.display_insurance_chip(triple=True)
+                    self.player.stack += (self.dealer.insurance_bet * 3)
+                else:
+                    self.hide_insurance_chip()
                 while self.dealer.is_finished is False:
                     self.dealer.deal(self.shoe, self.gui.shoe_progress)
                     self.display_dealer_cards()
@@ -210,30 +225,35 @@ class Game:
         """Handles payout of all hands."""
         self.hide_fingers()
         for hand in self.player.hands:
-            if hand.is_blackjack is True and self.dealer.is_blackjack is False:
+            if self.dealer.even_money is True:
+                self.player.stack += hand.bet * 2
+                result = f'EVEN MONEY'
+                self._display_chips(hand)
+            elif hand.is_blackjack is True and self.dealer.is_blackjack is False:
                 self.player.stack += hand.bet * 2.5
-                result = f'WIN BY BLACKJACK!'
+                result = f'BLACKJACK'
                 self._display_chips(hand, bj=True)
             elif hand.is_blackjack is True and self.dealer.is_blackjack is True:
                 self.player.stack += hand.bet
                 result = f'PUSH'
+            elif self.dealer.is_blackjack is True and hand.is_blackjack is False:
+                self.dealer_info('BLACKJACK')
+                result = f'LOSE'
+                self._resolve_lost_hand(hand)
             elif hand.is_over is False and self.dealer.is_over is True:
+                self.dealer_info('BUST')
                 self.player.stack += hand.bet * 2
-                result = f'WIN'
+                result = f''
                 self._display_chips(hand)
             elif hand.is_over is True:
                 result = f'BUST'
-                self.hide_chips(hand)
-                self.hide(hand)
-                hand.bet = 0
+                self._resolve_lost_hand(hand)
             elif hand.sum < self.dealer.sum:
                 result = f'LOSE ({hand.sum} vs {self.dealer.sum})'
-                self.hide_chips(hand)
-                self.hide(hand)
-                hand.bet = 0
+                self._resolve_lost_hand(hand)
             elif hand.surrender is True:
                 self.player.stack += hand.bet / 2
-                result = f'Lose by surrender'
+                result = ''
             elif hand.sum > self.dealer.sum:
                 self.player.stack += hand.bet * 2
                 result = f'WIN ({hand.sum} vs {self.dealer.sum})'
@@ -248,15 +268,18 @@ class Game:
         self.show_buttons(('deal',))
         self.gui.slider.configure(state=tkinter.NORMAL)
 
+    def _resolve_lost_hand(self, hand: Hand):
+        self.hide_chips(hand)
+        self.hide(hand)
+        hand.bet = 0
+
     def resolve_blackjack(self):
         """Resolves player blackjack."""
-        if self.dealer.cards[0].label == 'A' or self.dealer.cards[0].value == 10:
-            self.display_dealer_cards(hide_second=False)
+        self.display_dealer_cards(hide_second=False)
         self.payout()
 
     def enable_correct_buttons(self, hand: Hand):
         """Enables buttons that are OK to press with certain hand."""
-        self.clean_info()
         n_hands = len(self.player.hands)
         if len(hand.cards) == 2 and hand.is_hittable is True:
             self.show_buttons(('double',))
@@ -336,8 +359,9 @@ class Game:
     def show_buttons(self, buttons: tuple = None):
         """Shows menu buttons."""
         if buttons is None:
-            for button in self.gui.menu.values():
-                button.configure(state=tkinter.NORMAL)
+            for key, button in self.gui.menu.items():
+                if key not in ('insurance', 'even-money'):
+                    button.configure(state=tkinter.NORMAL)
         else:
             for button in buttons:
                 if button in self.gui.menu.keys():
@@ -374,8 +398,7 @@ class Game:
         for ind, card in enumerate(hand.cards):
             rotate = True if ind == len(hand.cards) - 1 and rotate_last is True else False
             img, width, height = get_image(card, rotate=rotate)
-            self.gui.slot_player[f'{str(hand.slot)}{str(ind)}'].configure(image=img, width=width,
-                                                                          height=height)
+            self.gui.slot_player[f'{str(hand.slot)}{str(ind)}'].configure(image=img, width=width, height=height)
             self.gui.slot_player[f'{str(hand.slot)}{str(ind)}'].image = img
 
     def display_player_hands(self):
@@ -384,6 +407,23 @@ class Game:
         for hand in self.player.hands:
             self.display_player_cards(hand)
 
+    def display_insurance_chip(self, triple: bool = False):
+        bet = self.dealer.insurance_bet if triple is False else self.dealer.insurance_bet * 3
+        color = 'red'
+        if bet == 0.5:
+            color = 'blue'
+            text = '0.5'
+        elif bet % 1 == 0:
+            text = str(round(bet))
+        else:
+            text = str(bet)
+        img = get_chip_image(color)
+        self.gui.insurance_chip.configure(image=img, compound='center', fg='white', text=text, font='helvetica 10 bold')
+        self.gui.insurance_chip.image = img
+
+    def hide_insurance_chip(self):
+        self.gui.insurance_chip.configure(image='', text='')
+
     def display_chip(self, hand: Hand, pos: int, color: str = 'red'):
         """Displays chip for certain hand and chip position."""
         img = get_chip_image(color)
@@ -391,8 +431,7 @@ class Game:
             text = self.bet
         else:
             text = '.5' if self.bet == 1 else self.bet / 2
-        self.gui.chips[f'{str(hand.slot)}{str(pos)}'].configure(image=img, compound='center',
-                                                                fg='white', text=text,
+        self.gui.chips[f'{str(hand.slot)}{str(pos)}'].configure(image=img, compound='center', fg='white', text=text,
                                                                 font='helvetica 10 bold')
         self.gui.chips[f'{str(hand.slot)}{str(pos)}'].image = img
 
@@ -403,6 +442,9 @@ class Game:
         self.gui.finger[f'{str(hand.slot)}'].configure(image=img)
         self.gui.finger[f'{str(hand.slot)}'].image = img
 
+    def dealer_info(self, text: str = ''):
+        self.gui.dealer_info.configure(text=text)
+
     def hide_chips(self, hand: Hand):
         """Hides chips of a hand."""
         for pos in range(4):
@@ -495,35 +537,36 @@ def main(args):
     root.title('Blackjack')
     root.configure(background=bc)
 
-    rect = tkinter.Canvas(root, bg=bc, height=100, width=80, bd=0, highlightthickness=0,
-                          relief='ridge')
+    rect = tkinter.Canvas(root, bg=bc, height=100, width=80, bd=0, highlightthickness=0, relief='ridge')
     rect.place(x=525, y=485)
     round_polygon(rect, [5, 75, 75, 5], [5, 5, 90, 90], 10, width=4, outline="#bbb500", fill=bc)
 
     # Shoe status
     shoe_status_container = tkinter.Label(root, borderwidth=0, background='white')
     shoe_status_container.place(x=20, y=30, height=150, width=30)
-    shoe_progress = tkinter.Label(shoe_status_container, background="black", borderwidth=0,
-                                  anchor="e")
-    shoe_label = tkinter.Label(root, text='Discard', font=12, borderwidth=0, background=bc,
-                               fg='white')
+    shoe_progress = tkinter.Label(shoe_status_container, background="black", borderwidth=0, anchor="e")
+    shoe_label = tkinter.Label(root, text='Discard', font=12, borderwidth=0, background=bc, fg='white')
     shoe_label.place(x=5, y=190)
 
     # Stack info
     label_text = tkinter.StringVar(root)
-    label = tkinter.Label(root, textvariable=label_text, font='Helvetica 13 bold', borderwidth=0,
-                          background=bc, fg='white')
+    label = tkinter.Label(root, textvariable=label_text, font='Helvetica 13 bold', borderwidth=0, background=bc,
+                          fg='white')
     label.place(x=430, y=670)
 
     # Hand info
     x_slot = 250
     padding_left = 20
     info_text = {str(slot): tkinter.StringVar(root) for slot in range(4)}
-    info = {str(slot): tkinter.Label(root, textvariable=info_text[str(slot)], font='helvetica 11 bold',
-                                     borderwidth=0, background=bc, fg="white")
+    info = {str(slot): tkinter.Label(root, textvariable=info_text[str(slot)], font='helvetica 11 bold', borderwidth=0,
+                                     background=bc, fg="white")
             for slot in range(4)}
     for ind, i in enumerate(info.values()):
-        i.place(x=ind*x_slot+padding_left+118, y=465)
+        i.place(x=ind*x_slot+padding_left+110, y=465)
+
+    # Dealer info
+    dealer_info = tkinter.Label(root, text='', font='helvetica 11 bold', borderwidth=0, background=bc, fg="white")
+    dealer_info.place(x=305, y=180)
 
     # Dealer finger
     finger = {str(slot): tkinter.Label(root, borderwidth=0, background=bc) for slot in range(4)}
@@ -535,14 +578,12 @@ def main(args):
                    for slot in range(4) for pos in range(N_CARDS_MAX)}
     for frame in range(4):
         for pos in range(N_CARDS_MAX):
-            slot_player[f'{str(frame)}{str(pos)}'].place(x=frame*x_slot+pos*30+padding_left,
-                                                         y=350-pos*30)
+            slot_player[f'{str(frame)}{str(pos)}'].place(x=frame*x_slot+pos*30+padding_left, y=350-pos*30)
 
     # Dealer cards
     n_dealer_cards = 7
     card_back_img, width_card, _ = get_image()
-    slot_dealer = {f'{str(pos)}': tkinter.Label(root, borderwidth=0, background=bc)
-                   for pos in range(n_dealer_cards)}
+    slot_dealer = {f'{str(pos)}': tkinter.Label(root, borderwidth=0, background=bc) for pos in range(n_dealer_cards)}
     for pos in range(2):
         slot_dealer[str(pos)].configure(image=card_back_img)
         slot_dealer[str(pos)].image = card_back_img
@@ -568,20 +609,22 @@ def main(args):
                 pady = 35
             chips[f'{str(slot)}{str(pos)}'].place(x=slot*x_slot+padding_left+padx+20, y=500+pady)
 
+    # Insurance chip
+    insurance_chip = tkinter.Label(root, borderwidth=0, background=bc)
+    insurance_chip.place(x=450, y=400)
+
     # Side panel
-    panel = tkinter.Label(root, width=200, height=720, background='lightgrey', borderwidth=2,
-                          relief="groove")
+    panel = tkinter.Label(root, width=200, height=720, background='lightgrey', borderwidth=2, relief="groove")
     panel.place(x=1000, y=0)
 
     # Advisor button
     fix_mistakes = tkinter.IntVar()
-    checkbox_container = tkinter.Checkbutton(root, text='Coach mode', variable=fix_mistakes,
-                                             background='lightgrey')
+    checkbox_container = tkinter.Checkbutton(root, text='Coach mode', variable=fix_mistakes, background='lightgrey')
     checkbox_container.place(x=1040, y=600)
 
     # Buttons
-    menu = {name.split()[0].lower(): tkinter.Button(master=root, text=name, width=12, font=15)
-            for name in ('Surrender', 'Double up', 'Hit', 'Stay', 'Split', 'Deal', 'Reset')}
+    menu = {name.split()[0].lower(): tkinter.Button(master=root, text=name.replace('-', ' '), width=12, font=15)
+            for name in ('Even-money', 'Insurance', 'Surrender', 'Double up', 'Hit', 'Stay', 'Split', 'Deal', 'Reset')}
     for name, button in menu.items():
         if name == 'hit':
             button.configure(command=lambda: game.hit())
@@ -597,19 +640,22 @@ def main(args):
             button.configure(command=lambda: game.next())
         elif name == 'reset':
             button.configure(command=lambda: game.reset())
+        elif name == 'insurance':
+            button.configure(command=lambda: game.insurance())
+        elif name == 'even-money':
+            button.configure(command=lambda: game.even_money())
         else:
             raise ValueError
     x_sidepanel = 1025
     for ind, button in enumerate(menu.values()):
-        button.place(x=x_sidepanel, y=ind*33+300)
+        button.place(x=x_sidepanel, y=ind*33+230)
 
     menu['deal'].place(x=x_sidepanel, y=500)
     menu['reset'].place(x=x_sidepanel, y=20)
 
     # Bet selector
     bet_label = tkinter.Label(text='Bet:', background='lightgray')
-    slider = tkinter.Scale(root, from_=1, to=10, orient=tkinter.HORIZONTAL,
-                           background='lightgray')
+    slider = tkinter.Scale(root, from_=1, to=10, orient=tkinter.HORIZONTAL, background='lightgray')
     slider.set(args.bet)
     slider.place(x=x_sidepanel+40, y=100)
     bet_label.place(x=x_sidepanel, y=120)
@@ -625,10 +671,12 @@ def main(args):
               finger,
               shoe_progress,
               fix_mistakes,
-              slider)
+              slider,
+              insurance_chip,
+              dealer_info)
 
     dealer = Dealer()
     player = Player(stack=args.stack)
-    game = Game(player, dealer, gui, bet=args.bet)
+    game = Game(player, dealer, gui, args)
     game.reset()
     tkinter.mainloop()
